@@ -1,5 +1,8 @@
 # Graphical display of the game world using GLFW and ModernGL
 # Copyright (C) 2019-2021  Nguyễn Gia Phong
+# Copyright (C) 2021  Ngô Ngọc Đức Huy
+# Copyright (C) 2021  Nguyễn Trường Giang
+# Copyright (C) 2021  Đặng Anh Đức
 #
 # This file is part of Axuy.
 #
@@ -21,6 +24,7 @@ __all__ = ['DispConfig', 'Display']
 
 from abc import abstractmethod
 from collections import deque
+from importlib.resources import open_binary, read_text
 from math import degrees, log2, radians
 from random import randint
 from statistics import mean
@@ -32,30 +36,28 @@ import numpy as np
 from PIL import Image
 from pyrr import matrix44
 
-from .misc import abspath, color, mirror
+from .misc import color, load_model, mirror
 from .peer import Peer, PeerConfig
-from .pico import OCTOVERTICES, SHARD_LIFE, TETRAVERTICES
+from .pico import RPICO, RSHARD, SHARD_LIFE
 
 CONWAY = 1.303577269034
 ABRTN_MAX = 0.42069
 
 QUAD = np.float32([-1, -1, 1, -1, -1, 1, -1, 1, 1, -1, 1, 1])
-TETRAINDECIES = np.int32([0, 1, 2, 3, 1, 2, 0, 3, 2, 0, 3, 1])
-OCTOINDECIES = np.int32([0, 1, 2, 0, 1, 3, 4, 0, 2, 4, 0, 3,
-                         2, 1, 5, 3, 1, 5, 2, 5, 4, 3, 5, 4])
+TETRAINDICES = np.int32([0, 2, 1, 3, 1, 2, 0, 3, 2, 0, 1, 3])
 
-with open(abspath('shaders/map.vert')) as f: MAP_VERTEX = f.read()
-with open(abspath('shaders/map.frag')) as f: MAP_FRAGMENT = f.read()
-with open(abspath('shaders/pico.vert')) as f: PICO_VERTEX = f.read()
-with open(abspath('shaders/pico.geom')) as f: PICO_GEOMETRY = f.read()
-with open(abspath('shaders/pico.frag')) as f: PICO_FRAGMENT = f.read()
+MAP_VERTEX = read_text('axuy.shaders', 'map.vert')
+MAP_FRAGMENT = read_text('axuy.shaders', 'map.frag')
+PICO_VERTEX = read_text('axuy.shaders', 'pico.vert')
+PICO_GEOMETRY = read_text('axuy.shaders', 'pico.geom')
+PICO_FRAGMENT = read_text('axuy.shaders', 'pico.frag')
 
-with open(abspath('shaders/tex.vert')) as f: TEX_VERTEX = f.read()
-with open(abspath('shaders/sat.frag')) as f: SAT_FRAGMENT = f.read()
-with open(abspath('shaders/gaussh.vert')) as f: GAUSSH_VERTEX = f.read()
-with open(abspath('shaders/gaussv.vert')) as f: GAUSSV_VERTEX = f.read()
-with open(abspath('shaders/gauss.frag')) as f: GAUSS_FRAGMENT = f.read()
-with open(abspath('shaders/comb.frag')) as f: COMBINE_FRAGMENT = f.read()
+TEX_VERTEX = read_text('axuy.shaders', 'tex.vert')
+SAT_FRAGMENT = read_text('axuy.shaders', 'sat.frag')
+GAUSSH_VERTEX = read_text('axuy.shaders', 'gaussh.vert')
+GAUSSV_VERTEX = read_text('axuy.shaders', 'gaussv.vert')
+GAUSS_FRAGMENT = read_text('axuy.shaders', 'gauss.frag')
+COMBINE_FRAGMENT = read_text('axuy.shaders', 'comb.frag')
 
 
 class DispConfig(PeerConfig):
@@ -192,26 +194,36 @@ class Display(Peer):
         self.fpses = deque()
 
         # Window's rendering and event-handling configuration
-        glfw.set_window_icon(self.window, 1, Image.open(abspath('icon.png')))
+        icon = Image.open(open_binary('axuy.assets', 'icon.png'))
+        glfw.set_window_icon(self.window, 1, icon)
         glfw.make_context_current(self.window)
         glfw.swap_interval(config.vsync)
         glfw.set_window_size_callback(self.window, self.resize)
 
         # Create OpenGL context
         self.context = context = moderngl.create_context()
-        context.enable_only(moderngl.DEPTH_TEST)
+        context.enable_only(moderngl.DEPTH_TEST | moderngl.CULL_FACE)
 
         # GLSL program and vertex array for map rendering
         self.maprog = context.program(vertex_shader=MAP_VERTEX,
                                       fragment_shader=MAP_FRAGMENT)
+        self.maprog['tex'], self.maprog['norm'] = 2, 3
         mapvb = context.buffer(mirror(self.space))
         self.mapva = context.simple_vertex_array(self.maprog, mapvb, 'in_vert')
 
+        # Wall texture
+        wall_color = Image.open(open_binary('axuy.assets', 'wall-color.png'))
+        context.texture(wall_color.size, 3, wall_color.tobytes()).use(2)
+        wall_norm = Image.open(open_binary('axuy.assets', 'wall-norm.png'))
+        context.texture(wall_norm.size, 4, wall_norm.tobytes()).use(3)
+
         # GLSL programs and vertex arrays for picos and shards rendering
-        pvb = [(context.buffer(TETRAVERTICES), '3f', 'in_vert')]
-        pib = context.buffer(TETRAINDECIES)
-        svb = [(context.buffer(OCTOVERTICES), '3f', 'in_vert')]
-        sib = context.buffer(OCTOINDECIES)
+        pv, pi = load_model('chickenV2.obj')
+        pvb = [(context.buffer(pv*RPICO), '3f', 'in_vert')]
+        pib = context.buffer(pi)
+        sv, si = load_model('rock01.obj')
+        svb = [(context.buffer(sv*RSHARD), '3f', 'in_vert')]
+        sib = context.buffer(si)
 
         self.prog = context.program(vertex_shader=PICO_VERTEX,
                                     geometry_shader=PICO_GEOMETRY,
@@ -236,8 +248,7 @@ class Display(Peer):
             self.gaussv, quad_buffer, 'in_vert')
         self.edge = context.program(vertex_shader=TEX_VERTEX,
                                     fragment_shader=COMBINE_FRAGMENT)
-        self.edge['la'].value = 0
-        self.edge['tex'].value = 1
+        self.edge['la'], self.edge['tex'] = 0, 1
         self.combine = context.simple_vertex_array(
             self.edge, quad_buffer, 'in_vert')
 
@@ -366,6 +377,7 @@ class Display(Peer):
 
         # Render map
         self.maprog['visibility'].value = visibility
+        self.maprog['camera'].write(self.pos)
         self.maprog['mvp'].write(vp)
         self.mapva.render(moderngl.TRIANGLES)
 
