@@ -18,11 +18,12 @@
 
 __doc__ = 'Axuy peer'
 __all__ = ['__version__', 'PeerConfig', 'Peer']
-__version__ = '0.0.11'
+__version__ = '0.0.12'
 
 from abc import ABC, abstractmethod
 from argparse import ArgumentParser, FileType, RawTextHelpFormatter
 from configparser import ConfigParser
+from importlib.resources import open_text, read_text
 from os.path import join as pathjoin, pathsep
 from pickle import dumps, loads
 from queue import Empty, Queue
@@ -33,11 +34,9 @@ from typing import Iterator, Tuple
 
 from appdirs import AppDirs
 
-from .misc import abspath, mapgen, mapidgen
+from .misc import mapgen, mapidgen
 from .pico import Pico
 
-SETTINGS = abspath('settings.ini')
-
 
 class PeerConfig:
     """Networking configurations.
@@ -64,7 +63,7 @@ class PeerConfig:
 
         # Parse configuration files
         self.config = ConfigParser()
-        self.config.read(SETTINGS)
+        self.config.read_file(open_text('axuy', 'settings.ini'))
         self.config.read(filenames)
         self.fallback()
 
@@ -102,12 +101,12 @@ class PeerConfig:
     @property
     def seeder(self) -> Tuple[str, int]:
         """Seeder address."""
-        return self.__seed
+        return self._seed
 
     @seeder.setter
     def seeder(self, value: str) -> None:
         host, port = value.split(':')
-        self.__seed = host, int(port)
+        self._seed = host, int(port)
 
     def read(self, arguments):
         """Read and parse a argparse.ArgumentParser.Namespace."""
@@ -119,7 +118,7 @@ class PeerConfig:
         """Parse all configurations."""
         args = self.options.parse_args()
         if args.cfgout is not None:
-            with open(SETTINGS) as f: args.cfgout.write(f.read())
+            args.cfgout.write(read_text('axuy', 'settings.ini'))
             args.cfgout.close()
             exit()
         if args.config:     # is neither None nor empty
